/* conf_window.cc: Config Window implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <iostream>
#include <limits>
#include <algorithm>

#include <libglademm/xml.h>

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/path.hpp"

#include "boost/bind.hpp"

#include "conf_window.hh"
#include "config.h"
#include "config.hh"

gx2osd::conf_window::conf_window()
    : boost::noncopyable(),
      xml_( Gnome::Glade::Xml::create( 
#ifdef PREFIX
                PREFIX "/share/gx2osd/gx2osd.glade"
#endif
                ) ),
      delay_( 0 ),
      positions_( 0 ),
      text_pos_( 0 ),
      custom_( 0 ),
      pos_box_( 0 ),
      x_pos_( 0 ),
      y_pos_( 0 ),
      skin_( 0 ),
      save_( 0 ),
      cancel_( 0 ),
      song_change_( 0 ),
      play_( 0 ),
      pause_( 0 ),
      stop_( 0 ),
      skin_names_(),
      skin_model_(),
      skin_list_( Gtk::ListStore::create( skin_model_ ) )
{

    load_widget( "conf_window", window_ );
    load_widget( "delay", delay_ );
    load_widget( "positions", positions_ );
    load_widget( "text_pos", text_pos_ );
    load_widget( "custom", custom_ );
    load_widget( "pos_box", pos_box_ );
    load_widget( "x_pos", x_pos_ );
    load_widget( "y_pos", y_pos_ );
    load_widget( "skins", skin_ );
    load_widget( "save", save_ );
    load_widget( "cancel", cancel_ );
    load_widget( "song_change", song_change_ );
    load_widget( "play", play_ );
    load_widget( "pause", pause_ );
    load_widget( "stop", stop_ );

    song_change_->set_active( get_config().get_song_change() );
    play_->set_active( get_config().get_play() );
    pause_->set_active( get_config().get_pause() );
    stop_->set_active( get_config().get_stop() );

    skin_->set_model( skin_list_ );
    skin_->pack_start( skin_model_.text );

    delay_->set_range( 0.0, std::numeric_limits< unsigned int >::max() );
    delay_->set_value( get_config().get_delay() );

    x_pos_->set_range( 0.0, std::numeric_limits< unsigned int >::max() );
    x_pos_->set_value( get_config().get_xpos() );

    y_pos_->set_range( 0.0, std::numeric_limits< unsigned int >::max() );
    y_pos_->set_value( get_config(). get_ypos() );

    if( get_config().get_position() != "" ) {
        positions_->set_active();
        text_pos_->set_active( get_config().get_index_of_position( get_config().get_position() ) );
        pos_box_->set_sensitive( false );
        text_pos_->set_sensitive( true );
    }
    else {
        custom_->set_active();
        pos_box_->set_sensitive( true );
        text_pos_->set_sensitive( false );
    }

    custom_->signal_toggled().connect
        ( sigc::mem_fun( *this, &gx2osd::conf_window::on_custom_radio_toggle ) );

    save_->signal_pressed().connect
        ( sigc::mem_fun( *this, &gx2osd::conf_window::on_save ) );
    cancel_->signal_pressed().connect
        ( sigc::mem_fun( *this, &gx2osd::conf_window::on_cancel ) );

    fs::path skin_path( get_config().get_config_dir() / "skins" );
    std::cout << "Looking for skins in '" << skin_path.string() << "'" << std::endl;
    if( fs::exists( skin_path ) ) {
        std::for_each( fs::directory_iterator( skin_path ),
                       fs::directory_iterator(),
                       boost::bind( &gx2osd::conf_window::skin_test, this, _1 ) );
    }

#ifdef PREFIX
    skin_path = PREFIX "/share/gx2osd/skins";
    std::cout << "Looking for skins in global directory '"
              << skin_path.string() << "'" << std::endl;
    std::for_each( fs::directory_iterator( skin_path ),
                   fs::directory_iterator(),
                   boost::bind( &gx2osd::conf_window::skin_test, this, _1 ) );
#endif

    std::for_each( skin_names_.begin(), skin_names_.end(),
                   boost::bind( &gx2osd::conf_window::add_skin, this, _1 ) );

    int index = std::distance( skin_names_.begin(),
                               skin_names_.find( get_config().get_skin_name() ) );
    skin_->set_active( index );

}

gx2osd::conf_window::~conf_window() {
}

Gtk::Window&
gx2osd::conf_window::get_window() {
    return *window_;
}

void
gx2osd::conf_window::on_custom_radio_toggle() {
    pos_box_->set_sensitive( !pos_box_->is_sensitive() );
    text_pos_->set_sensitive( !text_pos_->is_sensitive() );
}

void
gx2osd::conf_window::on_save() {

    fs::ofstream ofs( get_config().get_config_dir() / "gx2osd.conf" );
    ofs << "hide-delay = " << delay_->get_value_as_int() << "\n"
        << "position = ";
    if( positions_->get_active() ) {
        ofs << get_config().get_position( text_pos_->get_active_row_number() ) << "\n";
    }
    else {
        ofs << x_pos_->get_value_as_int() << " x " << y_pos_->get_value_as_int() << "\n";
    }

    ofs << "skin = " << (*skin_->get_active())[ skin_model_.text ] << "\n";
    ofs << std::boolalpha;
    ofs << "song-change = " << song_change_->get_active() << "\n"
        << "play = " << play_->get_active() << "\n"
        << "pause = " << pause_->get_active() << "\n"
        << "stop = " << stop_->get_active() << "\n";

    window_->hide();

}

void
gx2osd::conf_window::on_cancel() {
    window_->hide();
}

void
gx2osd::conf_window::skin_test( const fs::path& path ) {

    if( fs::is_directory( path ) ) {
        try {
            std::cout << "Trying to load skin: '" << path.leaf() << "'" << std::endl;
            gx2osd::skin test( path.leaf() );
            skin_names_.insert( path.leaf() );
            std::cout << std::endl;
        }
        catch( ... ) {
            std::clog << "Not a skin directory: " << path.string() << std::endl;
        }
    }

}

void
gx2osd::conf_window::add_skin( const Glib::ustring& skin ) {

    Gtk::TreeModel::Row row = *(skin_list_->append());
    row[ skin_model_.text ] = skin;

}
