/* config.cc: Configuration handling implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <xmmsclient/xmmsclient++.h>
#include "config.hh"

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/algorithm/string/case_conv.hpp"

#include <iostream>
#include <sstream>
#include <algorithm>
#include <iterator>

#include <signal.h>

gx2osd::config::config() 
    : noncopyable(),
      config_dir_( Xmms::getUserConfDir() + "/clients/gx2osd", fs::native ),
      config_file_( config_dir_ / "gx2osd.conf" ),
      pid_file_( config_dir_ / "gx2osd.pid" ),
      desc_( "gx2osd options" ),
      invis_( "Invisible options" ),
      all_( "Allowed options" ),
      vm_(),
      skin_name_( "default" ),
      skin_(),
      xpos_( 0 ),
      ypos_( 0 ),
      position_(),
      delay_( 3000 ),
      song_change_( true ),
      play_( true ),
      pause_( true ),
      stop_( true ),
      config_window_( false ),
      valid_positions_() {

    char const* const positions[] = { "top-left", "top-center", "top-right",
                                      "left-center", "center", "right-center", 
                                      "bottom-left", "bottom-center", "bottom-right" };
    std::copy( positions, positions + 9, std::back_inserter( valid_positions_ ) );

    if( !fs::exists( config_dir_ ) ) {
        fs::create_directory( config_dir_ );
    }

    desc_.add_options()
        ( "help,h", "Print this message." )
        ( "show", "Show already running OSD window." )
        ( "config", "Configure gx2osd" )
        ( "hide-delay,d",
          po::value< unsigned int >( &delay_ )->default_value( 3000 ),
          "Time to wait in ms before hiding the OSD window." )
        ( "position,p", 
          po::value< std::string >()->default_value("0 x 0")->notifier( 
              boost::bind(&gx2osd::config::parse_position, this, _1) ),
          "Main window position, either \"X x Y\" or one of the following:\n"
          "\"top-left\", \"top-center\", \"top-right\"\n"
          "\"left-center\", \"center\", \"right-center\"\n"
          "\"bottom-left\", \"bottom-center\", \"bottom-right\"." )
        ( "skin,s",
          po::value< std::string >( &skin_name_ )->default_value( "default" ),
          "skin to use." );

    invis_.add_options()
        ( "song-change", po::value< bool >( &song_change_ )->default_value( true ),
          "Show window on song change" )
        ( "play", po::value< bool >( &play_ )->default_value( true ),
          "Show window on play" )
        ( "pause", po::value< bool >( &pause_ )->default_value( true ),
          "Show window on pause" )
        ( "stop", po::value< bool >( &stop_ )->default_value( true ),
          "Show window on stop" );

    all_.add(desc_).add(invis_);

}

gx2osd::config::~config() {

}

gx2osd::config&
gx2osd::config::get_instance() {
    static config instance;
    return instance;
}

bool
gx2osd::config::set_args( const cmd_line& args ) {

    std::cout << "Parsing arguments and config file..." << std::endl;
    po::store( po::command_line_parser( args ).options( desc_ ).run(), vm_ );

    if( fs::exists( config_file_ ) ) {

        fs::ifstream ifs( config_file_ );
        po::store( po::parse_config_file( ifs, all_ ), vm_ );

    }

    po::notify( vm_ );
    if( vm_.count( "help" ) ) {
        std::cout << desc_ << std::endl;
        return false;
    }
    if( vm_.count( "show" ) ) {

        fs::ifstream ifs( pid_file_ );
        pid_t pid;
        ifs >> pid;
        kill( pid, SIGUSR1 );
        return false;

    }
    if( vm_.count( "config" ) ) {
        config_window_ = true;
        return false;
    }

    std::cout << "Loading skin: " << skin_name_ << std::endl;
    skin_.reset( new skin( skin_name_ ) );

    return true;

}

std::string const&
gx2osd::config::get_position() const {
    return position_;
}

std::string const&
gx2osd::config::get_position( positions_container::size_type pos ) const {
    return valid_positions_.at( pos );
}

gx2osd::config::positions_container::size_type
gx2osd::config::get_index_of_position( std::string const& pos ) const {
    return std::distance( valid_positions_.begin(),
                          std::find( valid_positions_.begin(), valid_positions_.end(),
                                     boost::algorithm::to_lower_copy( pos ) ) );
}

unsigned int
gx2osd::config::get_xpos() const {
    return xpos_;
}

unsigned int
gx2osd::config::get_ypos() const {
    return ypos_;
}

unsigned int
gx2osd::config::get_delay() const {
    return delay_;
}

fs::path
gx2osd::config::get_config_dir() const {
    return config_dir_;
}

const fs::path&
gx2osd::config::get_pid_file() const {
    return pid_file_;
}

const boost::shared_ptr< gx2osd::skin >&
gx2osd::config::get_skin() const {
    return skin_;
}

bool
gx2osd::config::show_conf() const {
    return config_window_;
}

const std::string&
gx2osd::config::get_skin_name() const {
    return skin_name_;
}

bool
gx2osd::config::get_song_change() const {
    return song_change_;
}

bool
gx2osd::config::get_play() const {
    return play_;
}

bool
gx2osd::config::get_pause() const {
    return pause_;
}

bool
gx2osd::config::get_stop() const {
    return stop_;
}

void
gx2osd::config::parse_position( const std::string& position ) {

    if( std::find( valid_positions_.begin(), valid_positions_.end(),
                   position ) != valid_positions_.end() ) {
        position_ = position;
    }
    else {
        std::istringstream ss( position );
        char temp;
        if( !(ss >> xpos_ >> temp >> ypos_ && temp == 'x') ) {
            std::cerr << "Invalid position " << position << ". "
                      << "Setting position to 0 x 0."
                      << std::endl;
            xpos_ = 0; ypos_ = 0;
        }
    }

}

gx2osd::config&
gx2osd::get_config() {
    return gx2osd::config::get_instance();
}
