/* main_window.cc: OSD Window implementation.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <gtk/gtk.h>
#include <gtkmm/window.h>
#include <gtkmm/label.h>
#include <gdkmm/screen.h>
#include <gdkmm/pixbuf.h>
#include <gdkmm/pixbufloader.h>
#include <gtkmm/image.h>
#include <glibmm/ustring.h>

#include "main_window.hh"
#include "config.hh"
#include "scroller.hh"

#include <iostream>

#include <xmmsclient/xmmsclient++.h>


gx2osd::main_window::main_window() 
    : Gtk::Window( Gtk::WINDOW_POPUP ),
      artist_name_(),
      song_name_(),
      fixed_pane_(),
      artist_markup_(),
      song_name_markup_(),
      span_end_( "</span>" ),
      cover_(),
      status_(),
      skin_( get_config().get_skin() ) {

    // TODO: find a way to do this in gtkmm... or prove not possible yet.
    gtk_window_set_accept_focus( GTK_WINDOW( gobj() ), FALSE );
    set_decorated( false );
    set_keep_above( true );
    set_events( Gdk::BUTTON_PRESS_MASK );

    fixed_pane_.add( background_ );

    load_skin();

    if( !get_config().get_position().empty() ) {

        gx2osd::config::positions_container::size_type pos =
            get_config().get_index_of_position( get_config().get_position() );

        int screen_width = get_screen()->get_width();
        int screen_height = get_screen()->get_height();
        Glib::RefPtr<Gdk::Pixbuf> buf( skin_->background() );
        int width = buf->get_width();
        int height = buf->get_height();
        switch( pos ) {
            case 0: {
                move( 0, 0 );
            } break;
            case 1: {
                move( (screen_width / 2 - width / 2), 0 );
            } break;
            case 2: {
                move( screen_width - width, 0 );
            } break;
            case 3: {
                move( 0, (screen_height / 2 - height / 2) );
            } break;
            case 4: {
                set_position( Gtk::WIN_POS_CENTER );
            } break;
            case 5: {
                move( screen_width - width,
                      (screen_height / 2 - height / 2) );
            } break;
            case 6: {
                move( 0, screen_height - height );
            } break;
            case 7: {
                move( (screen_width / 2 - width / 2),
                      screen_height - height );
            } break;
            case 8: {
                move( screen_width - width,
                      screen_height - height );
            } break;
        }

    }
    else {
        move( get_config().get_xpos(), get_config().get_ypos() );
    }

    fixed_pane_.put( artist_name_, skin_->get_artist().get_position().x,
                                   skin_->get_artist().get_position().y );
    fixed_pane_.put( song_name_, skin_->get_songname().get_position().x,
                                 skin_->get_songname().get_position().y );
    fixed_pane_.put( cover_, skin_->no_cover_image().get_position().x, 
                             skin_->no_cover_image().get_position().y );
    fixed_pane_.put( status_, skin_->get_status_position().x, 
                              skin_->get_status_position().y );

    add( fixed_pane_ );

}

gx2osd::main_window::~main_window() {

}

void
gx2osd::main_window::set_text( const Glib::ustring& artist,
                               const Glib::ustring& title,
                               const Glib::ustring& album ) {

    artist_name_.set_markup( artist_markup_ 
                           + Glib::Markup::escape_text( artist )
                           + span_end_ );

    song_name_.set_markup( song_name_markup_
                         + Glib::Markup::escape_text( title )
                         + span_end_ );

    if( skin_->has_album() ) {
        album_name_.set_markup( album_name_markup_
                              + Glib::Markup::escape_text( album )
                              + span_end_ );
    }

}

void
gx2osd::main_window::set_text_and_show( const Glib::ustring& artist,
                                        const Glib::ustring& title,
                                        const Glib::ustring& album ) {

    cover_.set( skin_->no_cover_image() );
    set_text( artist, title, album );
    if( get_config().get_song_change() ) {
        show_all();
    }

}

void
gx2osd::main_window::set_cover( const Xmms::bin& cover,
                                const Glib::ustring& artist,
                                const Glib::ustring& title,
                                const Glib::ustring& album ) {

    try {

        Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();

        loader->write( cover.data(), cover.size() );
        loader->close();

        cover_.set( scale_pixbuf( loader->get_pixbuf() ) );

    }
    catch ( Glib::Error& e ) {
        std::clog << "Could not load image: " << e.what() << std::endl;
        cover_.set( skin_->no_cover_image() );
    }

    set_text( artist, title, album );
    if( get_config().get_song_change() ) {
        show_all();
    }

}

bool
gx2osd::main_window::set_status( const Xmms::Playback::Status& status ) {
    if( set_status_no_show( status ) ) {
        show_all();
    }
    return true;
}

bool
gx2osd::main_window::set_status_no_show( const Xmms::Playback::Status& status ) {
    switch( status ) {
        case Xmms::Playback::STOPPED: {
            status_.set( skin_->stopped_image() );
            return get_config().get_stop();
            break;
        }
        case Xmms::Playback::PLAYING: {
            status_.set( skin_->playing_image() );
            return get_config().get_play();
            break;
        }
        case Xmms::Playback::PAUSED: {
            status_.set( skin_->paused_image() );
            return get_config().get_pause();
            break;
        }
    }
    return true;
}

void
gx2osd::main_window::show_all() {
    static sigc::connection conn;
    Gtk::Window::show_all();
    if( conn ) {
        conn.disconnect();
    }
    conn = Glib::signal_timeout().connect( sigc::mem_fun(*this,
                                           &main_window::hider),
                                           get_config().get_delay() );
}

bool
gx2osd::main_window::hider() {
    hide();
    return false;
}

void
gx2osd::main_window::load_skin() {

    set_label_stuff( artist_name_, skin_->get_artist(), artist_markup_ );
    set_label_stuff( song_name_, skin_->get_songname(), song_name_markup_ );

    if( skin_->has_album() ) {

        set_label_stuff( album_name_, skin_->get_album(), album_name_markup_ );
        fixed_pane_.put( album_name_, skin_->get_album().get_position().x,
                                      skin_->get_album().get_position().y );

    }
    else if( album_name_.get_parent() != 0 ) {
        fixed_pane_.remove( album_name_ );
    }

    Glib::RefPtr<Gdk::Pixbuf> buf( skin_->background() );

    if( buf ) {

        Glib::RefPtr<Gdk::Bitmap> mask;
        Glib::RefPtr<Gdk::Pixmap> pixmap;

        buf->render_pixmap_and_mask( pixmap, mask, 128 );

        resize( buf->get_width(), buf->get_height() );
        shape_combine_mask(mask, 0, 0);

        background_.set( buf );

    }
    
    cover_.set( skin_->no_cover_image() );
    status_.set( skin_->paused_image() );
    
}

void
gx2osd::main_window::set_label_stuff( eclidgets::scroller& gtklabel,
                                      const gx2osd::label& label,
                                      Glib::ustring& markup ) {

    Pango::FontDescription font = gtklabel.get_font_description();
    font.set_size( label.get_size() * Pango::SCALE );
    gtklabel.set_font_description( font );

    markup = "<span weight=\"";

    if( label.get_bold() ) {
        markup += "bold\"";
    }
    else {
        markup += "normal\"";
    }
    markup += " color=\"" + label.get_color() + "\">";

    gtklabel.set_size_request( label.get_width() );

}

bool
gx2osd::main_window::on_button_press_event( GdkEventButton* event )
{
    Gtk::Window::on_button_press_event( event );
    hide();
    return true;
}

Glib::RefPtr< Gdk::Pixbuf >
gx2osd::main_window::scale_pixbuf( Glib::RefPtr< Gdk::Pixbuf > const& pixbuf ) const
{
    const int width = pixbuf->get_width();
    const int height = pixbuf->get_height();
    int dest_width = 100;
    int dest_height = 100;
    double ratio = width / static_cast< double >(height);

    if( width > height ) {
        dest_height = static_cast< int >(100 / ratio);
    }
    else if( height > width ) {
        dest_width = static_cast< int >(100 * ratio);
    }

    return pixbuf->scale_simple( dest_width, dest_height, Gdk::INTERP_BILINEAR );
}
