/* scroller.cc: Scrolling text widget implementation
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include "scroller.hh"
#include <gtkmm/settings.h>
#include <iostream>
#include <cmath>

eclidgets::scroller::scroller( Glib::ustring const& text, 
                               scrolling scroll, scroll_direction direction,
                               Gtk::Justification default_justify,
                               unsigned int interval, unsigned int multiplier,
                               unsigned int wait_before_scroll,
                               int gap )
    : Gtk::Widget(), layout_( create_pango_layout( text ) ), gc_(),
      pos_( 0 ), justify_( default_justify ),
      scroll_( scroll ), direction_( direction ), scrolling_( false ),
      width_( 0 ), interval_( interval ), multiplier_( multiplier ),
      wait_before_scroll_( wait_before_scroll ), gap_( gap ),
      conn_(), wait_()
{
    Pango::FontDescription font( Gtk::Settings::get_default()->property_gtk_font_name().get_value() );
    if( layout_ ) {
        layout_->set_font_description( font );
    }
    set_flags( Gtk::NO_WINDOW );
}

eclidgets::scroller::~scroller()
{
}

void
eclidgets::scroller::set_text( Glib::ustring const& str )
{
    if( layout_ ) {
        layout_->set_text( str );
        text_changed();
    }
}

Glib::ustring
eclidgets::scroller::get_text() const
{
    return layout_->get_text();
}

eclidgets::scroller::scrolling
eclidgets::scroller::get_scroll() const
{
    return scroll_;
}

eclidgets::scroller::scroll_direction
eclidgets::scroller::get_direction() const
{
    return direction_;
}

Gtk::Justification
eclidgets::scroller::get_justification() const
{
    return justify_;
}

void
eclidgets::scroller::get_speed( unsigned int& interval, unsigned int& multiplier ) const
{
    interval = interval_;
    multiplier = multiplier_;
}

int
eclidgets::scroller::get_gap() const
{
    return gap_;
}

unsigned int
eclidgets::scroller::get_wait_before_scroll() const
{
    return wait_before_scroll_;
}

Pango::FontDescription
eclidgets::scroller::get_font_description() const
{
    return layout_->get_font_description();
}

void
eclidgets::scroller::set_markup( Glib::ustring const& markup )
{
    layout_->set_markup( markup );
    text_changed();
}

void
eclidgets::scroller::set_scroll( scrolling scroll )
{
    scroll_ = scroll;
    determine_scroll();
}

void
eclidgets::scroller::set_direction( scroll_direction direction )
{
    direction_ = direction;
}

void
eclidgets::scroller::set_justification( Gtk::Justification justify )
{
    justify_ = justify;
    determine_scroll();
    queue_draw();
}

void
eclidgets::scroller::set_speed( unsigned int interval, unsigned int multiplier )
{
    interval_ = interval;
    multiplier_ = multiplier;
    if( conn_ ) {
        conn_.disconnect();
        determine_scroll();
    }
}

void
eclidgets::scroller::set_gap( int gap )
{
    gap_ = gap;
}

void
eclidgets::scroller::set_width_chars( unsigned int chars )
{
    Glib::RefPtr< Pango::Context > context = layout_->get_context();
    int char_width = context->get_metrics( context->get_font_description() ).get_approximate_char_width();
    int height = 0;
    int width = 0;
    layout_->get_pixel_size( width, height );
    set_size_request( static_cast< int >(char_width * chars / Pango::SCALE * 1.2),
                      height );
}

void
eclidgets::scroller::set_wait_before_scroll( unsigned int wait_before_scroll )
{
    wait_before_scroll_ = wait_before_scroll;
}

void
eclidgets::scroller::set_font_description( Pango::FontDescription const& desc )
{
    if( layout_ ) {
        layout_->set_font_description( desc );
    }
}

void
eclidgets::scroller::on_realize()
{
    Gtk::Widget::on_realize();
    int height = 0;
    int width = 0;
    layout_->get_pixel_size( width, height );
    set_size_request( get_width(), height );
    determine_scroll();
}

bool
eclidgets::scroller::on_expose_event( GdkEventExpose* event )
{
    Gdk::Rectangle alloc = get_allocation();
    int x = alloc.get_x();
    int y = alloc.get_y();
    int w = get_width();

    Gtk::Widget::on_expose_event( event );
    if( !gc_ ) {
        gc_ = get_style()->get_fg_gc( get_state() );
    }
    Gdk::Rectangle clip( x, y, alloc.get_width(), alloc.get_height() );
    gc_->set_clip_rectangle( clip );

    get_window()->draw_layout( gc_, pos_ + x, 0 + y, layout_ );

    if( scrolling_ && direction_ == SCROLL_TO_LEFT ) {
        if( (pos_ < 0 && width_ < w) ) {
            get_window()->draw_layout( gc_, w + pos_ + x, 0 + y, layout_ );
            if( std::abs( pos_ ) > width_ ) {
                pos_ = w - width_;
            }
        }
        else if( (width_ > (w - gap_) && width_ - w + pos_ + gap_ < 0) ) {
            get_window()->draw_layout( gc_, width_ + pos_ + gap_ + x, 0 + y, layout_ );
            if( std::abs( pos_ ) > (width_ + gap_) ) {
                pos_ = 0;
            }
        }
    }
    else if( scrolling_ && direction_ == SCROLL_TO_RIGHT ) {

        if( (width_ < w && pos_ + width_ > w) ) {
            get_window()->draw_layout( gc_, pos_ - w + x, 0 + y, layout_ );
            if( pos_ > w ) {
                pos_ = 0;
            }
        }
        else if( (width_ > (w - gap_) && pos_ > gap_) ) {
            get_window()->draw_layout( gc_, pos_ - width_ - gap_ + x, 0 + y, layout_ );
            if( pos_ > (w + gap_) ) {
                pos_ = w - width_;
            }
        }

    }
    return true;
}

void
eclidgets::scroller::on_size_allocate( Gtk::Allocation& allocation )
{
    Gtk::Widget::on_size_allocate( allocation );
    determine_scroll();
}

bool
eclidgets::scroller::timer()
{
    pos_ += direction_ * multiplier_;
    queue_draw();
    return true;
}

void
eclidgets::scroller::determine_scroll()
{

    if( !is_realized() ) {
        return;
    }

    bool would_scroll = width_ > get_width();

    if( !would_scroll || scroll_ == SCROLL_NEVER ) {
        switch( justify_ ) {
            case Gtk::JUSTIFY_LEFT: {
                pos_ = 0;
            } break;
            case Gtk::JUSTIFY_RIGHT: {
                pos_ = get_width() - width_;
            } break;
            default: {
                pos_ = (get_width() - width_) / 2;
            }
        }
    }

    if( (would_scroll || scroll_ == SCROLL_ALWAYS) ) {
        scrolling_ = true;
        if( !conn_ ) {
            if( width_ > get_width() ) {
                if( direction_ == SCROLL_TO_RIGHT ) {
                    pos_ = get_width() - width_;
                }
                else {
                    pos_ = 0;
                }
            }
            if( wait_ ) {
                wait_.disconnect();
            }
            wait_ = Glib::signal_timeout().
                        connect( sigc::mem_fun( *this, &scroller::start_scroll ),
                                 wait_before_scroll_ );
        }
    }
    else {
        if( wait_ ) {
            wait_.disconnect();
        }
        if( conn_ ) {
            conn_.disconnect();
        }
        scrolling_ = false;
        queue_draw();
    }

}

bool
eclidgets::scroller::start_scroll()
{
    conn_ = Glib::signal_timeout()
        .connect( sigc::mem_fun( *this, &scroller::timer ), interval_ );
    return false;
}

void
eclidgets::scroller::text_changed()
{
    int height = 0;
    layout_->get_pixel_size( width_, height );

    conn_.disconnect();
    determine_scroll();
    queue_draw();
}
